from textual.theme import Theme

SOLARIZED_DARK = Theme(
    name="solarized-dark",
    primary="#268bd2",
    secondary="#2aa198",
    accent="#dc322f",
    foreground="#839496",
    background="#002b36",
    success="#859900",
    warning="#b58900",
    error="#dc322f",
    surface="#073642",
    panel="#073642",
    dark=True,
)

CATPPUCCIN_MOCHA = Theme(
    name="catppuccin-mocha",
    primary="#cba6f7",
    secondary="#89b4fa",
    accent="#f38ba8",
    foreground="#cdd6f4",
    background="#1e1e2e",
    success="#a6e3a1",
    warning="#f9e2af",
    error="#f38ba8",
    surface="#313244",
    panel="#313244",
    dark=True,
)
